/*
BoxedWine Application.

BoxedWine (C) boxedwine.org
BoxedWine for Win96 (C) windows96.net
*/

const { register } = w96.app;
const { Theme, DialogCreator, MsgBoxSimple } = w96.ui;

const DEFAULT_CONFIG = {
    net: {
        ver_repo: "https://packages.windows96.net/main/dist/w/wine/versions"
    },
    wine: {
        version: "wine17",
        shell: "C:\\windows\\explorer.exe"
    }
}

class BoxedWineApplication extends WApplication {
    constructor() {
        super();
        this.blobsToUnload = [];
    }

    async mkblob(path) {
        const u = URL.createObjectURL(await FS.toBlob(path));
        this.blobsToUnload.push(u);
        return u;
    }
    
    async main(argv) {
        super.main(argv);

        // Setup FS
        if(!(await FS.exists("c:/user/appdata/bWINE")))
            await FS.mkdir("c:/user/appdata/bWINE");

        // Show first boot dialog if needed
        if(!(await FS.exists("c:/user/appdata/bWINE/.first_boot"))) {
            await alert(`Welcome to Wine for Windows 96 (bWINE).<br>
<br>
Since this is your first time running bWINE, here are some important things you need to know:<br><br>
 - It will be slow, since BoxedWine (the backend) is emulating Wine through a Linux emulator.<br>
 - Wine 5.0 will esepcially take its time to boot.<br>
 - Different versions of Wine are available through the version selector.<br>
 - bWINE is currently experimental. A number of features will be missing.<br>
 - Take general Wine-isms into account (the usual problems with Wine under Linux)`, {icon:"info"}).wait();
            await FS.touch("c:/user/appdata/bWINE/.first_boot");
        }

        // Read config and create it if it doesnt exist.
        if(await FS.exists("c:/user/appdata/bWINE/config.json")) {
            this.config = {
                ...DEFAULT_CONFIG,
                ...JSON.parse(await FS.readstr("c:/user/appdata/bWINE/config.json"))
            }
        } else {
            this.config = { ... DEFAULT_CONFIG };
            await FS.writestr("c:/user/appdata/bWINE/config.json", JSON.stringify(this.config, null, 4));
        }

        // Check command line arguments

        if(argv.includes("--manage-versions"))
            return alert("version manager WIP");
        else if(argv.includes("--about"))
            return DialogCreator.create({
                title: "About bWINE",
                body: `<span class="bold-noaa">bWINE</span><br>Version 1.0<br><br>Uses BoxedWine, which can be found at <a href="#" onclick="w96.sys.execCmd('iexploder', ['http://boxedwine.org'])">boxedwine.org</a><br>
                Also uses WINE, which can be found at <a href="#" onclick="w96.sys.execCmd('iexploder', ['https://winehq.org'])">winehq.org</a>`,
                icon: "info" 
            });

        // Fetch versions and save download URL
        const statusUi = MsgBoxSimple.idleProgress("bWINE Init", "Fetching version information...");

        const v_resp = await fetch(this.config.net.ver_repo + "/index.json");
        const v_available = await v_resp.json();

        statusUi.closeDialog();

        const v_verObj = v_available.find(x => x.id == this.config.wine.version);

        // Check if config specified version exists.
        if(!v_verObj) {
            await alert(`Wine version in config not found. Press OK to close bWINE.`, { icon: "error" }).wait();
            return;
        }
        
        // Initialize Main UI

        const mainwnd = this.createWindow({
            title: "WINE",
            icon: Theme.getIconUrl("exec", "small"),
            initialHeight: 621,
            initialWidth: 800,
            body: `<iframe></iframe>
            <style>
                .boxed-wine-app>iframe { border: none; height: 100%; width: 100%; }
            </style>`,
            bodyClass: "boxed-wine-app",
            center: true,
            taskbar: true
        }, true); // true specifies that this is an app window (main window)
        
        const body = mainwnd.getBodyContainer();
        const appFrame = body.querySelector('iframe');

        appFrame.src = await this.mkblob("c:/local/bwine/res/frame.html");
        mainwnd.setWindowIcon(await this.mkblob("c:/local/bwine/res/icon.png"));

        appFrame.onload = async ()=>{
            appFrame.contentDocument.onclick = ()=>appFrame.focus();

            // Load ZIP
            // appFrame.contentWindow.zipUrl = await this.mkblob("c:/local/bwine/versions/default.zip"); // TODO allow change
            // Program name
            appFrame.contentWindow.verBase = this.config.net.ver_repo;
            appFrame.contentWindow.zipName = v_verObj.file;
            appFrame.contentWindow.urlParams = "p=" + ((argv[0] == null) ? this.config.wine.shell : argv[0]);

            const SCRIPTS = [
                "c:/local/bwine/scripts/browserfs.boxedwine.js",
                "c:/local/bwine/scripts/jszip.min.js",
                "c:/local/bwine/scripts/boxedwine-shell.js"
            ];

            const aDoc = appFrame.contentDocument;

            for(let script of SCRIPTS) {
                const scriptEl = aDoc.createElement("script");
                scriptEl.src = await this.mkblob(script);
                aDoc.body.appendChild(scriptEl);
            }

            // Modify the main script a little to inject the WASM blob
            const wasmBlob = await this.mkblob("c:/local/bwine/bin/bwine.wasm");
            let mainScript = await FS.readstr("c:/local/bwine/scripts/boxedwine.js");
            mainScript = mainScript.replace(/\%BWINE_WASM_FILE\%/g, wasmBlob);

            const scriptEl = document.createElement('script');
            scriptEl.async = true;
            scriptEl.innerText = mainScript;
            aDoc.body.appendChild(scriptEl);
        };

        mainwnd.show();
    }

    ontermination() {
        super.ontermination();

        for(let b of this.blobsToUnload)
            URL.revokeObjectURL(b);
    }
}

register({
    command: "bwine",
    type: "gui",
    cls: BoxedWineApplication,
    meta: {
        icon: Theme.getIconUrl("exec"),
        friendlyName: "Wine"
    }
});